#ifdef __USAGE		/* nbs.c */
%C	[-cmpq]
Options:
  -c           Don't set the clock with date and time
  -m ser_port  Device to use (default: /dev/ser1)
  -p prefix    Prefix to the NBS phone number (maybe "9,")
  -q           Quiet (do not display local time)
  -v           Verbose (also display the UTC time)
Note:
  If -c (Don't set clock) is specified, time of day will be displayed 15 times
#endif

/* Author: Terry Laughlin                                         April, 1992
 *
 * This is a program to get the UTC time from the National Bureau of Standards
 * in Colorado.  All I had was a phone number, so if you know more about the
 * structure, feel free to improve.  Only one caveat, I freely give it into the
 * public domain without restrictions and I assume no responsibility for the
 * results (make sure it hangs up your phone!).
 *
 * The strings put out by the NBS computer look like:

48728 92-04-16 07:06:09 50 0 -.4 045.0 UTC(NIST) *
48728 92-04-16 07:06:10 50 0 -.4 045.0 UTC(NIST) *
48728 92-04-16 07:06:11 50 0 -.4 045.0 UTC(NIST) *
48728 92-04-16 07:06:12 50 0 -.4 045.0 UTC(NIST) *

 * I can see the date, time and UTC(NIST) constant.  The asterisk is the tone
 * character, as in 'at the sound of the tone, the time will be...'.  The number
 * up front (48728) appears to be a date and the rest hasn't changed in the
 * last week.  But I could see all I needed...
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <ctype.h>
#include <fcntl.h>
#include <string.h>
#include <termios.h>
#include <time.h>
#include <sys/timers.h>
#include <sys/dev.h>

#define old_mode()	{ dev_mode(mdm, omdm_mode, _DEV_MODES); }
#define new_mode()	{ dev_mode(mdm,  mdm_mode, _DEV_MODES); }

#define VERSION				"NBS for QNX 4"
#define NBS_PHONE			"1,303-494-4774\r"
#define BUFFER_SIZE			50
#define MAXLOOPS			15

extern void	 gotsig();
extern char	*optarg;
extern int	 optind;

struct nbs
	{
	char	 num1[5];
	char	 space1;
	char	 year[2];
	char	 dash1;
	char	 mon[2];
	char	 dash2;
	char	 mday[2];
	char	 eodate;
	char	 hour[2];
	char	 colon1;
	char	 min[2];
	char	 colon2;
	char	 sec[2];
	char	 eotime;
	char	 fifty[2];
	char	 space2;
	char	 zero;
	char	 space3;
	char	 num2[3];
	char	 space4;
	char	 num3[5];
	char	 space5;
	char	 UTC[3];
	char	 oparen;
	char	 NIST[4];
	char	 cparen;
	} nbs;

struct _dev_info_entry	mdm_entry;
struct tm		tm, *ptm;
struct timespec	ts;
time_t			time_of_day;

int		 gottime;
int		 settime = 1;
int		 mdm;
int		 quiet = 0;
int		 verbose = 0;
unsigned mdm_mode, omdm_mode;

char	*mname = "/dev/ser1",
		 buf[ BUFFER_SIZE ],
		*prefix;


void gotsig( int sig )
	{
	all_done( sig );
	}


main( int argc, char *argv[] )
	{
	unsigned opt;
	int		 n;
	int		 loop = 0;

	while( ( opt = getopt( argc, argv, "cm:p:qv" ) ) != -1 )
		{
		switch( opt )
			{
			case 'c':	settime = 0;				break;
			case 'm':	mname = optarg;				break;
			case 'p':	prefix = optarg;			break;
			case 'q':	quiet = 1;					break;
			case 'v':	verbose = 1;				break;
			default:	exit( EXIT_FAILURE );		break;
			}
		}

	if( verbose == 0 && quiet == 1 && settime == 0 )
		quiet = 0;						/* should do something... */

	if( quiet == 0 || verbose )
		printf( "%s:   At the '*', the time will be...\n\n", VERSION );

	mdm = open( mname, O_RDWR );
	if ( mdm == -1 )
		{
		fprintf(stderr, "%s: Unable to open device %s\n", argv[0], mname );
		exit( 1 );
		}

	dev_info( mdm, &mdm_entry );
	if( mdm_entry.open_count > 1 )
		{
		fprintf(stderr, "%s: Device %s in use.  Try again later.\n", argv[0], mname );
		exit(1);
		}

	signal( SIGINT, &gotsig );
	signal( SIGQUIT, &gotsig );
	signal( SIGTERM, &gotsig );
										/* Set modem completely RAW and flush */
	omdm_mode = dev_mode( mdm, (mdm_mode = 0), _DEV_MODES);
	tcflush( mdm, TCIOFLUSH );

	sleep(1);
	tcflush( mdm, TCIFLUSH );

	putout( "ATTD" );
	if( prefix && *prefix )
		putout( prefix );
	putout( NBS_PHONE );

	for( n = 0; n < 6; ++n )			/* 60 seconds to start spewing chars */
		{
		if( dev_read( mdm, buf, sizeof(buf), sizeof(buf), 0, 100, 0, 0) == sizeof(buf) )
			break;
		}

	for( loop = (settime) ? MAXLOOPS : 0, n = 0; n < 30; ++n )
		{
		for( buf[0] = 0; buf[0] != '*'; )
			{
			if( dev_read( mdm, buf, 1, 1, 0, 5, 0, 0) < 1 )
				break;
			}

		if( gottime && buf[0] == '*' )
			{
			if( settime )
				setclock( TIMEOFDAY, &ts );

			if( quiet == 0 || verbose )
				{
				putc( '*', stdout );
				fflush( stdout );
				putc( '\n', stdout );
				}

			if( loop++ >= MAXLOOPS )
				{
				if( settime )
					fprintf( stdout, "\nuse rtc to set the hardware clock\n" );
				all_done( 0 );
				}
			}

		gottime = 0;					/* reset flag */

		for( buf[0] = 0; buf[0] != 0x0a /* linefeed */; )
			{
			if( dev_read( mdm, buf, 1, 1, 0, 5, 0, 0) < 1 )
				break;
			}

		if( dev_read( mdm, &nbs, sizeof(nbs), sizeof(nbs), 0, 15, 0, 0) == sizeof(nbs) )
			{
			if( nbs.space1 == ' ' && nbs.dash1 == '-'
						&& nbs.dash2 == '-' && nbs.eodate == ' '
						&& nbs.colon1 == ':' && nbs.colon2 == ':'
						&& nbs.eotime == ' ' && nbs.space2 == ' '
						&& nbs.space3 == ' ' && nbs.space4 == ' '
						&& nbs.space5 == ' ' && strncmp(nbs.UTC,"UTC",3) == 0
						&& nbs.oparen == '(' && strncmp(nbs.NIST,"NIST",4) == 0
						&& nbs.cparen == ')' )
				{
				++gottime;

				nbs.dash1  = nbs.dash2  = nbs.eodate = 0;
				nbs.colon1 = nbs.colon2 = nbs.eotime = 0;

				tm.tm_year = atoi( nbs.year );
				tm.tm_mon  = atoi( nbs.mon ) - 1;
				tm.tm_mday = atoi( nbs.mday );
				tm.tm_hour = atoi( nbs.hour );
				tm.tm_min  = atoi( nbs.min );
				tm.tm_sec  = atoi( nbs.sec );
				ts.tv_sec = mktime( &tm );
				ts.tv_sec -= timezone;

				if( verbose )
					{
					fprintf( stdout, "UTC time: %02d:%02d:%02d   date: %02d/%02d/%02d  ",
							tm.tm_hour, tm.tm_min, tm.tm_sec, tm.tm_mon+1, tm.tm_mday, tm.tm_year );
					}
				if( quiet == 0 )
					{
					if( verbose )
						putc( '\n', stdout );
					ptm = localtime( &ts.tv_sec );
					fprintf( stdout, "   Local: %02d:%02d:%02d   date: %02d/%02d/%02d ",
							ptm->tm_hour, ptm->tm_min, ptm->tm_sec, ptm->tm_mon+1, ptm->tm_mday, ptm->tm_year );
					}
				fflush( stdout );
				}
			}
		}
	fprintf( stderr, "%s: Could not get time\n", argv[0] );
	all_done( 1 );
	}


all_done( int rc )
	{
	tcdropline( mdm, 500 );
	sleep( 1 );
	old_mode();
	exit( rc );
	}


putout( char *str )
	{
	char	*p;

	for( p = str; *p; ++p )
		{
		tcflush( mdm, TCIFLUSH );
		write( mdm, p, 1 );
		}
	}
